#include <lib/libsa/stand.h>
#include <lib/libkern/libkern.h>

#include <sys/cdefs.h>
#include <sys/types.h>

#define MIPS3 1
#include <uvm/uvm_extern.h>
#include <mips/cpuregs.h>
#include <mips/pte.h>
extern void mips3_TLBRead __P((int, struct tlb *));
extern int mips3_GetWIRED __P((void));

#if 0
#include <arc/arc/arcbios.h>
#else
#include "arcbios.h"
#endif

#include <stdio.h>
#include "dumpconf.h"

#define ARRAY_LENGTH(array)	(sizeof(array)/sizeof(array[0]))

void dump_argv(FILE *file, int argc, char **argv);
void dump_environ(FILE *file, char **envp);
void dump_tlb_lo(FILE *file, unsigned int);
void dump_tlb(FILE *file);
void dump_all(FILE *file, int argc, char **argv, char **envp);
int parse_command(char *command, char **cmdv, int maxcmdc);
void __main(void);
int main(int argc, char **argv, char **envp);

void
dump_argv(FILE *file, int argc, char **argv)
{
	int i;

	fprintf(file, "argc = %d\n", argc);
	for (i = 0; i < argc; i++)
		fprintf(file, "argv[%d] = `%s'\n", i, argv[i]);
}

void
dump_environ(FILE *file, char **envp)
{
	while (*envp)
		fprintf(file, "[%s]\n", *envp++);
}

void
dump_tlb_lo(FILE *file, unsigned int tlb_lo)
{
	if (tlb_lo & 0xfc000000) {
		fprintf(file, "0x%x ", tlb_lo >> 26);
	} else {
		fprintf(file, "0x");
	}
	fprintf(file, "%x(%d%c%c%c)", (tlb_lo & 0x03ffffc0) << 6,
		(tlb_lo & MIPS3_TLB_ATTR_MASK) >> MIPS3_TLB_ATTR_SHIFT,
		(tlb_lo & MIPS3_TLB_DIRTY_BIT) ? 'D' : '.',
		(tlb_lo & MIPS3_TLB_VALID_BIT) ? 'V' : '.',
		(tlb_lo & MIPS3_TLB_GLOBAL_BIT) ? 'G' : '.');
}

void
dump_tlb(FILE *file)
{
	int i;

	fprintf(file, "wired TLB = %d\n", mips3_GetWIRED());
	for (i = 0; i < MIPS3_TLB_NUM_TLB_ENTRIES; i++) {
		struct tlb tlb;

		mips3_TLBRead(i, &tlb);

		fprintf(file, "%s%d: ", i < 10 ? " " : "", i);
		switch (tlb.tlb_mask & MIPS3_PG_SIZE_16M) {
		case MIPS3_PG_SIZE_4K:
			fprintf(file, "4K");
			break;
		case MIPS3_PG_SIZE_16K:
			fprintf(file, "16K");
			break;
		case MIPS3_PG_SIZE_64K:
			fprintf(file, "64K");
			break;
		case MIPS3_PG_SIZE_256K:
			fprintf(file, "256K");
			break;
		case MIPS3_PG_SIZE_1M:
			fprintf(file, "1M");
			break;
		case MIPS3_PG_SIZE_4M:
			fprintf(file, "4M");
			break;
		case MIPS3_PG_SIZE_16M:
			fprintf(file, "16M");
			break;
		default:
			fprintf(file, "unknown(0x%x)",
				tlb.tlb_mask & MIPS3_PG_SIZE_16M);
			break;
		}

		fprintf(file, ", asid: 0x%x, v: 0x%x, ",
			tlb.tlb_hi & MIPS3_PG_ASID,
			tlb.tlb_hi & MIPS3_PG_HVPN);
		if (tlb.tlb_hi & ~(MIPS3_PG_HVPN|MIPS3_PG_ASID))
			fprintf(file, ", rfuhi: 0x%x, ",
				tlb.tlb_hi & ~(MIPS3_PG_HVPN|MIPS3_PG_ASID));
		fprintf(file, "p0: ");
		dump_tlb_lo(file, tlb.tlb_lo0);
		fprintf(file, ", p1: ");
		dump_tlb_lo(file, tlb.tlb_lo1);
		fprintf(file, "\n");
	}
}

void
dump_all(FILE *file, int argc, char **argv, char **envp)
{
	fprintf(file, "Command Arguments:\n");
	dump_argv(file, argc, argv);
	fprintf(file, "\nEnvironment Variables:\n");
	dump_environ(file, envp);
	fprintf(file, "\nSystem Parameter Block:\n");
	bios_dump_param_blk(file);
	fprintf(file, "\nSystem Identifier:\n");
	bios_dump_system_id(file);
	fprintf(file, "\nSystem Configuration Tree:\n");
	bios_dump_configuration(file);
	fprintf(file, "\nMemory Layout:\n");
	bios_dump_memory(file);
	fprintf(file, "\nDisplay Status:\n");
	bios_dump_display_status(file, 1);
#if 0 /* PICA-61 sometimes misbehaves, if write-count isn't multple of 512 */
	fflush(file); /* dump_tlb() depends on R4000/R4400 */
#endif
	fprintf(file, "\nTLB:\n");
	dump_tlb(file);
}

int parse_command(char *command, char **cmdv, int maxcmdc)
{
	int cmdc = 0;

	while (*command) {
		while (*command == ' ' || *command == '\t')
			command++;
		if (*command == '\0')
			break;
		if (cmdc >= maxcmdc)
			break;
		cmdv[cmdc++] = command;
		while (*command && *command != ' ' && *command != '\t')
			command++;
		if (*command)
			*command++ = '\0';
	}
	return (cmdc);
}

void
__main()
{}


int
main(argc, argv, envp)
	int argc;
	char **argv;
	char **envp;
{
	int i;
	char buffer[100];
	int cmdc;
	char *cmdv[10];
	static char dumpfilename[80] =
		"scsi()disk(0)rdisk()partition(1)\\arcdiag.out";
	static char dumpfileext[] = ".out";

	if (!bios_magic())
		return (1);

	if (strlen(argv[0]) + sizeof(dumpfileext) <= sizeof(dumpfilename)) {
		strcpy(dumpfilename, argv[0]);
		strcat(dumpfilename, dumpfileext);
	}
	printf("default dumpfile = `%s'\n", dumpfilename);

	printf("argc:\t%i\n",argc);
	printf("argv:\t%p\n",argv);
	printf("envp:\t%p\n",envp);

	dump_all(stdout, argc, argv, envp);

	return (0);
}
